@@ -32,6 +32,7 @@ int gengetopt_add_option (const char * long_opt, char short_opt,
                           const char *default_value,
                           const char * group_value, 
                           const char * type_str,
-                          int multiple);
+                          int multiple = 0,
+                          int argoptional = 0);
 
 #endif /* _GENGETOPT_H */
