@@ -25,7 +25,8 @@ struct gengetopt_option
   char * default_string ; /* default value for this option, if string */
   char * group_value; /* group name, if it's part of an option group */
   char * group_desc; /* group description, if it's part of an option group */
-  int multiple; /* whether this option can be given more than once */
+  bool multiple; /* whether this option can be given more than once */
+  bool arg_is_optional; /* whether the argument is optional */
   char *type_str; /* Alternative name for type,
                      e.g. "URL" or "SECONDS" */
   char *section; /* the section of this option */
