@@ -42,7 +42,7 @@ extern "C"
 
 #include "groups.h"
 #include "string_opt_arg.h"
-#include "string_opt_arg_free.h"
+#include "string_opt_arg_no_free.h"
 #include "int_opt_arg.h"
 #include "long_opt_arg.h"
 #include "long_long_opt_arg.h"
@@ -59,14 +59,21 @@ extern "C"
 #include "handle_unamed.h"
 #include "print_group.h"
 #include "config_parser_header.h"
+#include "config_parser_source.h"
 #include "multiple_opt_list.h"
+#include "multiple_opt_struct.h"
 #include "multiple_option.h"
 #include "multiple_option_no_short.h"
+#include "multiple_option_new_list_node.h"
 #include "multiple_fill_array.h"
 #include "multiple_fill_array_default.h"
 #include "gen_strdup.h"
-
-#include "conf_parser_gen.h"
+#include "free_string.h"
+#include "free_multiple_string.h"
+#include "free_inputs_string.h"
+#include "reset_group.h"
+#include "exit_failure.h"
+#include "multiple_option_token_functions.h"
 
 #ifndef HAVE_STRDUP
 extern "C" char *strdup (const char *s) ;
@@ -97,6 +104,12 @@ generate_help_desc_print(ostream &stream,
 			 unsigned int desc_column,
 			 const char *descript, const char *defval);
 
+static bool
+has_multiple_options_string();
+
+static bool
+has_multiple_options();
+
 char *
 create_filename (char *name, char *ext)
 {
@@ -130,8 +143,9 @@ open_fstream (char *filename)
   return fstream;
 }
 
-CmdlineParserCreator::CmdlineParserCreator (char *function_name, 
-                                            bool unamed_options_,
+CmdlineParserCreator::CmdlineParserCreator (char *function_name,
+                                            char *struct_name,
+                                            char *unamed_options_,
                                             char *filename_, 
                                             char *header_ext, char *c_ext,
                                             bool long_help_, 
@@ -139,13 +153,15 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
                                             bool no_handle_version_,
                                             bool no_handle_error_,
                                             bool conf_parser_,
+                                            bool gen_version,
                                             const string &comment_) :
-  filename (filename_), 
+  filename (filename_),
+  args_info_name (struct_name),
   comment (comment_),
   unamed_options (unamed_options_), 
   long_help (long_help_), no_handle_help (no_handle_help_),
   no_handle_version (no_handle_version_), no_handle_error (no_handle_error_),
-  conf_parser (conf_parser_),
+  conf_parser (conf_parser_), gen_gengetopt_version (gen_version),
   tab_indentation (0)
 {
   parser_function_name = canonize_names (function_name);
@@ -155,7 +171,9 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
   // header_gen_class
   const string stripped_header_file_name = strip_path (filename);
   set_header_file_name (stripped_header_file_name);
-  header_gen_class::set_generator_version (VERSION);
+  if (gen_gengetopt_version)
+    header_gen_class::set_generator_version 
+      ("version " VERSION);
   const string my_ifndefname = 
     to_upper (strip_path (stripped_header_file_name));
   set_ifndefname (canonize_names (my_ifndefname.c_str ()));
@@ -168,6 +186,8 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
   c_source_gen_class::set_package_var_name (my_package_var_name);
   header_gen_class::set_version_var_name (my_version_var_name);
   c_source_gen_class::set_version_var_name (my_version_var_name);
+  header_gen_class::set_args_info (args_info_name);
+  c_source_gen_class::set_args_info (args_info_name);
   const string uppersand = "\"";
 
   if (gengetopt_package)
@@ -188,6 +208,7 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
       ostringstream conf_parser_proto;
       config_parser_header_gen_class cf_gen;
       cf_gen.set_parser_name (parser_function_name);
+      cf_gen.set_args_info (args_info_name);
       conf_parser_proto << endl;
       cf_gen.generate_config_parser_header (conf_parser_proto);
       conf_parser_proto << endl;
@@ -196,11 +217,51 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
 
   // c_source_gen_class
   set_command_line (comment);
-  c_source_gen_class::set_generator_version (VERSION);
+  if (gen_gengetopt_version)
+    c_source_gen_class::set_generator_version
+      ("version " VERSION);
   c_source_gen_class::set_parser_name (parser_function_name);
   set_source_name (filename);
 
-  set_gen_exit (no_handle_error ? "return" : "exit");
+  if (! no_handle_error)
+    {
+      ostringstream exit_failure_str;
+      exit_failure_gen_class exit_gen;
+      exit_gen.set_parser_name (c_source_gen_class::parser_name);
+
+      exit_gen.generate_exit_failure (exit_failure_str);
+
+      set_final_exit (exit_failure_str.str ());
+    }
+  else
+    set_final_exit ("return result;");
+
+  struct gengetopt_option * opt;
+  gen_strdup = (unamed_options != 0 || conf_parser);
+
+  if (! gen_strdup)
+    {
+      foropt
+	if (opt->type == ARG_STRING) {
+          gen_strdup = true;
+          break;
+	}
+    }
+
+  if (gen_strdup)
+    set_generated_strdup 
+      ("static char *\ngengetopt_strdup (const char *s);\n");
+
+  if (has_multiple_options())
+    {
+      ostringstream multi_functions_stream;
+
+      multiple_option_token_functions_gen_class multi_functions;
+      multi_functions.generate_multiple_option_token_functions (multi_functions_stream);
+
+      set_multiple_token_functions (multi_functions_stream.str ());
+      set_multiple_token_vars ("\n  char *multi_token, *multi_next; /* for multiple options */");
+    }
 }
 
 void
@@ -216,12 +277,16 @@ CmdlineParserCreator::generateBreak(ostream &stream, unsigned int indent)
 void
 CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
                                      ostream &stream,
-                                     unsigned int indent) 
+                                     unsigned int indent,
+                                     const string argstr = "optarg" )
 {
-  const string argstr ("optarg");
+  string indent_str (indent, ' ');
+  const int offset = 4;
+
   string_opt_arg_gen_class str_gen;
   str_gen.set_optarg (argstr);
-  string_opt_arg_free_gen_class str_gen_free;
+  string_opt_arg_no_free_gen_class str_gen_no_free;
+  str_gen_no_free.set_optarg (argstr);
   int_opt_arg_gen_class int_gen;
   int_gen.set_optarg (argstr);
   long_opt_arg_gen_class long_gen;
@@ -235,7 +300,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
   if (opt->multiple)
     {
       int_gen.set_structure (string (opt->var_arg) + "_new");
-      str_gen.set_structure (string (opt->var_arg) + "_new");
+      str_gen_no_free.set_structure (string (opt->var_arg) + "_new");
       long_gen.set_structure (string (opt->var_arg) + "_new");
       long_long_gen.set_structure (string (opt->var_arg) + "_new");
       float_gen.set_structure (string (opt->var_arg) + "_new");
@@ -244,12 +309,19 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
     {
       int_gen.set_structure (ARGS_STRUCT);
       str_gen.set_structure (ARGS_STRUCT);
-      str_gen_free.set_structure (ARGS_STRUCT);
       long_gen.set_structure (ARGS_STRUCT);
       long_long_gen.set_structure (ARGS_STRUCT);
       float_gen.set_structure (ARGS_STRUCT);
     }
 
+  if (opt->arg_is_optional)
+    {
+      stream << "if (" << argstr << ")" << endl;
+      stream << indent_str << "  {" << endl;
+      stream << indent_str << "    ";
+      indent += offset;
+    }
+
   switch (opt->type)
     {
     case ARG_NO:
@@ -258,64 +330,65 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
     case ARG_FLAG:
       flag_gen.set_opt_var (opt->var_arg);
       flag_gen.generate_flag_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_STRING:
-      str_gen.set_opt_var (opt->var_arg);
-      if (! opt->multiple && opt->default_given)
+      if (opt->multiple)
         {
-          str_gen_free.set_opt_var (opt->var_arg);
-          str_gen_free.generate_string_opt_arg_free (stream, indent);
+          str_gen_no_free.set_opt_var (opt->var_arg);
+          str_gen_no_free.generate_string_opt_arg_no_free (stream, indent);
+        }
+      else
+        {
+          str_gen.set_opt_var (opt->var_arg);
+          str_gen.generate_string_opt_arg (stream, indent);
         }
-      str_gen.generate_string_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_INT:
       int_gen.set_cast ("");
       int_gen.set_opt_var (opt->var_arg);
       int_gen.generate_int_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_SHORT:
       int_gen.set_cast ("(short)");
       int_gen.set_opt_var (opt->var_arg);
       int_gen.generate_int_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_LONG:
       long_gen.set_cast ("");
       long_gen.set_opt_var (opt->var_arg);
       long_gen.generate_long_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_FLOAT:
       float_gen.set_cast ("(float)");
       float_gen.set_opt_var (opt->var_arg);
       float_gen.generate_float_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_DOUBLE:
       float_gen.set_cast ("");
       float_gen.set_opt_var (opt->var_arg);
       float_gen.generate_float_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_LONGDOUBLE:
       float_gen.set_cast ("(long double)");
       float_gen.set_opt_var (opt->var_arg);
       float_gen.generate_float_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     case ARG_LONGLONG:
       long_long_gen.set_opt_var (opt->var_arg);
       long_long_gen.generate_long_long_opt_arg (stream, indent);
-      generateBreak(stream, indent);
       break;
     default:
       fprintf (stderr, "gengetopt: bug found in %s:%d\n", __FILE__,
                __LINE__);
       abort ();
     }
+
+  if (opt->arg_is_optional)
+    {
+      stream << endl;
+      stream << indent_str << "  }";
+      indent -= offset;
+    }
 }
 
 int
@@ -603,7 +676,7 @@ CmdlineParserCreator::generate_usage_string()
     }
 
   if ( unamed_options )
-    usage << " [FILES]...";
+    usage << " [" << unamed_options << "]...";
 
   usage << "\\n";
 
@@ -755,7 +828,7 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
       first_option = false;
 
       stream << indent_str;
-      stream << "printf(\"";
+      stream << "printf(\"%s\\n\",\"";
       if (opt->type == ARG_FLAG || opt->type == ARG_NO)
         {
 	  const char * def_val = NULL;
@@ -774,7 +847,7 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
 	    def_val = opt->flagstat ? "on" : "off";
 	  
 	  generate_help_desc_print(stream, desc_col, opt->desc, def_val);
-          stream << "\\n\");\n";
+          stream << "\");\n";
         }
       else
         {
@@ -799,8 +872,14 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
 	      col += 6;
             }
 
-          stream << "--" << opt->long_opt << "=" << type_str;
+          bool arg_optional = opt->arg_is_optional;
+          stream << "--" << opt->long_opt 
+                 << (arg_optional ? "[" : "")
+                 << "=" << type_str
+                 << (arg_optional ? "]" : "");
 	  col += 3 + strlen(opt->long_opt) + type_len;
+          if (arg_optional)
+            col += 2;
 	  
 	  string desc_indent (desc_col - col, ' ');
 	  stream << desc_indent;
@@ -813,7 +892,7 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
 	    }
 
 	  generate_help_desc_print(stream, desc_col, opt->desc, def_val);
-          stream << "\\n\");\n";
+          stream << "\");\n";
         }
     }
 }
@@ -821,18 +900,6 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
 void
 CmdlineParserCreator::generate_strdup(ostream &stream, unsigned int indent)
 {
-  struct gengetopt_option * opt;
-  bool gen_strdup = unamed_options;
-
-  if (! gen_strdup)
-    {
-      foropt
-	if (opt->type == ARG_STRING) {
-          gen_strdup = 1;
-          break;
-	}
-    }
-
   stream << endl;
   if (gen_strdup) 
     {
@@ -857,14 +924,118 @@ CmdlineParserCreator::generate_given_init(ostream &stream,
       stream << ARGS_STRUCT << "->" << opt->var_arg << "_given = 0 ;";
       stream << endl;
     }
+
+  groups_collection_t::const_iterator end = gengetopt_groups.end();
+  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
+        idx != end; ++idx)
+    {
+      stream << indent_str;
+      stream << ARGS_STRUCT << "->" << canonize_name (idx->first) << "_group_counter = 0 ;";
+      stream << endl;
+    }
 }
 
+void
+CmdlineParserCreator::generate_reset_groups(ostream &stream, unsigned int indent)
+{
+  struct gengetopt_option * opt;
+  string indent_str (indent, ' ');
+  ostringstream body;
+  reset_group_gen_class reset_group;
+
+  reset_group.set_args_info (c_source_gen_class::args_info);
+
+  groups_collection_t::const_iterator end = gengetopt_groups.end();
+  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
+        idx != end; ++idx)
+    {
+      reset_group.set_name (canonize_name (idx->first));
+      body.str ("");
+
+      foropt
+      {
+        if (opt->group_value && strcmp(opt->group_value, idx->first.c_str()) == 0)
+          {
+            /* now we reset "given" fields */
+            body << indent_str;
+            body << ARGS_STRUCT << "->" << opt->var_arg << "_given = 0 ;";
+            body << endl;
+            
+            free_option (opt, body, indent);
+          }
+      }
+
+      reset_group.set_body (body.str ());
+      reset_group.generate_reset_group (stream);
+    }
+}
+
+void
+CmdlineParserCreator::free_option(struct gengetopt_option *opt, 
+                                  ostream &stream, unsigned int indent)
+{
+  if (! opt->type)
+    return;
+
+  if (opt->type == ARG_STRING)
+    {
+      if (opt->multiple)
+        {
+          free_multiple_string_gen_class free_multiple_string;
+          free_multiple_string.set_comment ("previous argument");
+          free_multiple_string.set_structure (ARGS_STRUCT);
+          
+          free_multiple_string.set_opt_var (opt->var_arg);
+          free_multiple_string.generate_free_multiple_string
+            (stream, indent);
+        }
+      else
+        {
+          free_string_gen_class free_string;
+          free_string.set_comment ("previous argument");
+          free_string.set_structure (ARGS_STRUCT);
+
+          free_string.set_opt_var (opt->var_arg);
+          free_string.generate_free_string (stream, indent);
+        }
+    }
+  else if (opt->multiple)
+    {
+      free_string_gen_class free_string;
+      free_string.set_comment ("previous argument");
+      free_string.set_structure (ARGS_STRUCT);
+
+      // it's not really a string, but an array anyway
+      free_string.set_opt_var (opt->var_arg);
+      free_string.generate_free_string (stream, indent);
+    }
+}
+
+
 void
 CmdlineParserCreator::generate_struct_def(ostream &stream, unsigned int indent)
+{
+  struct gengetopt_option * opt;
+  multiple_opt_struct_gen_class multiple_opt_struct;
+
+  /* define structs for multiple options */
+  foropt
+    {
+      if (opt->multiple && opt->type)
+	{
+	  multiple_opt_struct.set_type (arg_types[opt->type]);
+	  multiple_opt_struct.set_arg_name (opt->var_arg);
+	  multiple_opt_struct.generate_multiple_opt_struct (stream, 0);
+	}
+    }
+}
+
+void
+CmdlineParserCreator::generate_list_def(ostream &stream, unsigned int indent)
 {
   struct gengetopt_option * opt;
   string indent_str (indent, ' ');
-  multiple_opt_list_gen_class multiple_opt;
+  multiple_opt_list_gen_class multiple_opt_list;
   bool generated_counter = false;
 
   /* define linked-list structs for multiple options */
@@ -878,10 +1049,12 @@ CmdlineParserCreator::generate_struct_def(ostream &stream, unsigned int indent)
               stream << "int i;        /* Counter */" << endl;
               generated_counter = true;
             }
-          stream << indent_str;
-	  multiple_opt.set_type (arg_types[opt->type]);
-	  multiple_opt.set_arg_name (opt->var_arg);
-	  multiple_opt.generate_multiple_opt_list (stream, indent);
+          if (opt->type)
+            {
+              stream << indent_str;
+              multiple_opt_list.set_arg_name (opt->var_arg);
+              multiple_opt_list.generate_multiple_opt_list (stream, indent);
+            }
           stream << endl;
 	}
     }
@@ -897,7 +1070,7 @@ CmdlineParserCreator::generate_multiple_fill_array(ostream &stream, unsigned int
   /* copy linked list into the array */
   foropt
     {
-      if (opt->multiple)
+      if (opt->multiple && opt->type)
 	{
 	  stream << indent_str;
 	  filler.set_type (arg_types[opt->type]);
@@ -932,7 +1105,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
   /* now we initialize "given" fields */
   foropt
     {
-      if (opt->multiple)
+      if (opt->multiple && opt->type)
         {
           // since multiple options are stored in arrays, we
           // always initialize it to NULL
@@ -940,7 +1113,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
           stream << indent_str;
 
           stream << ARGS_STRUCT << "->" << opt->var_arg 
-                 << "_arg = NULL; \\" << endl;
+                 << "_arg = NULL;" << endl;
         }
       else if (opt->type == ARG_STRING)
         {
@@ -948,11 +1121,11 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
 
           if (opt->default_given)
             stream << ARGS_STRUCT << "->" << opt->var_arg 
-                   << "_arg = gengetopt_strdup(\"" << opt->default_string 
-                   << "\") ;\\";
+                   << "_arg = gengetopt_strdup (\"" << opt->default_string 
+                   << "\");";
           else
             stream << ARGS_STRUCT << "->" << opt->var_arg 
-                   << "_arg = NULL; \\";
+                   << "_arg = NULL;";
 
           stream << endl;
         }
@@ -961,7 +1134,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
           stream << indent_str;
 
           stream << ARGS_STRUCT << "->" << opt->var_arg << "_flag";
-          stream << " = " << opt->flagstat << ";\\";
+          stream << " = " << opt->flagstat << ";";
 
           stream << endl;
         }
@@ -970,7 +1143,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
           stream << indent_str;
 
           stream << ARGS_STRUCT << "->" << opt->var_arg << "_arg";
-          stream << " = " << opt->default_string << " ;\\";
+          stream << " = " << opt->default_string << " ;";
 
           stream << endl;
         }
@@ -1008,7 +1181,8 @@ CmdlineParserCreator::generate_long_option_struct(ostream &stream,
       stream << indent_str;
       
       stream << "{ \"" << opt->long_opt << "\",\t" 
-             << (opt->type == ARG_NO || opt->type == ARG_FLAG ? 0 : 1) 
+             << (opt->type == ARG_NO || opt->type == ARG_FLAG ? 0 : 
+                 (opt->arg_is_optional ? 2 : 1)) 
              << ", NULL, ";
 
       if (opt->short_opt)
@@ -1028,8 +1202,12 @@ CmdlineParserCreator::generate_getopt_string()
 
   foropt
     if (opt->short_opt)
-      built_getopt_string << opt->short_opt <<
-        (opt->type == ARG_NO || opt->type == ARG_FLAG ? "" : ":");
+      {
+        built_getopt_string << opt->short_opt <<
+          (opt->type == ARG_NO || opt->type == ARG_FLAG ? "" : ":");
+        built_getopt_string << 
+          (opt->arg_is_optional ? ":" : "");
+      }
 
   return built_getopt_string.str ();
 }
@@ -1045,7 +1223,6 @@ CmdlineParserCreator::generate_handle_help(ostream &stream,
      help_gen.set_short_option (HELP_SHORT_OPT_STR);
      help_gen.set_option_comment (HELP_OPT_DESCR);
      help_gen.set_option_var_name (HELP_LONG_OPT);
-     help_gen.set_gen_exit ("return");
      help_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
      help_gen.generate_generic_option (stream, indent);
@@ -1072,7 +1249,6 @@ CmdlineParserCreator::generate_handle_version(ostream &stream,
      version_gen.set_short_option (VERSION_SHORT_OPT_STR);
      version_gen.set_option_comment (VERSION_OPT_DESCR);
      version_gen.set_option_var_name (VERSION_LONG_OPT);
-     version_gen.set_gen_exit ("return");
      version_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
      version_gen.generate_generic_option (stream, indent);
@@ -1092,12 +1268,15 @@ void
 CmdlineParserCreator::generate_handle_no_short_option(ostream &stream, 
                                                       unsigned int indent)
 {
+  const string tmpvar("multi_token"); // reuse the `c' variable (returning getopt_long value)
+  const string optarg("optarg");
+
   struct gengetopt_option * opt;
   no_short_option_gen_class opt_gen;
   string indent_str (indent, ' ');
-  opt_gen.set_gen_exit (no_handle_error ? "return" : "exit");
   opt_gen.set_package_var_name (c_source_gen_class::package_var_name);
   multiple_option_no_short_gen_class multip_opt_gen;
+  multiple_option_new_list_node_gen_class multi_gen_node;
   bool first = true;
 
   foropt
@@ -1108,21 +1287,35 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
 
         if (opt->multiple)
           {
+            ostringstream list_stream;
+
             multip_opt_gen.set_option_comment (opt->desc);
             multip_opt_gen.set_long_option (opt->long_opt);
             multip_opt_gen.set_option_var_name (opt->var_arg);
 
-            do_update_arg (opt, str_stream, indent + 2);
-            multip_opt_gen.set_update_arg (str_stream.str ());
+            if (opt->type)
+              { // arguments possibly exists
+                do_update_arg (opt, str_stream, indent + 2, tmpvar);
+
+                multi_gen_node.set_optarg(optarg);
+                multi_gen_node.set_option_var_name (opt->var_arg);
+                multi_gen_node.set_update_arg(str_stream.str());
+                multi_gen_node.generate_multiple_option_new_list_node (list_stream);
+              }
+            
+            multip_opt_gen.set_update_arg ("break;");
+
+            multip_opt_gen.set_new_node_list (list_stream.str ());
 
             generic_option_group_gen_class group_gen;
             if (opt->group_value)
               {
                 str_stream.str ("");
-                str_stream << " ";
+                str_stream << endl;
+                str_stream << "  ";
                 group_gen.set_group_var_name 
                   (canonize_name (opt->group_value));
-                group_gen.generate_generic_option_group (str_stream, indent);
+                group_gen.generate_generic_option_group (str_stream, 2);
                 multip_opt_gen.set_update_group_count (str_stream.str ());
               }
             else
@@ -1137,17 +1330,18 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
             opt_gen.set_long_option (opt->long_opt);
             opt_gen.set_option_var_name (opt->var_arg);
 
-            do_update_arg (opt, str_stream, indent + 2);
+            do_update_arg (opt, str_stream, 0);
             opt_gen.set_update_arg (str_stream.str ());
 
             generic_option_group_gen_class group_gen;
             if (opt->group_value)
               {
                 str_stream.str ("");
-                str_stream << " ";
+                str_stream << endl;
+                str_stream << "  ";
                 group_gen.set_group_var_name 
                   (canonize_name (opt->group_value));
-                group_gen.generate_generic_option_group (str_stream, indent);
+                group_gen.generate_generic_option_group (str_stream, 2);
                 opt_gen.set_update_group_count (str_stream.str ());
               }
             else
@@ -1166,6 +1360,11 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
           }
       }
 
+  if (! first) // something has been generated
+    {
+      generateBreak(stream, indent);
+    }
+
   stream << endl;
 }
 
@@ -1173,18 +1372,17 @@ void
 CmdlineParserCreator::generate_handle_option(ostream &stream, 
                                              unsigned int indent)
 {
+  const string tmpvar("multi_token"); // reuse the `c' variable (returning getopt_long value)
+  const string optarg("optarg");
+
   struct gengetopt_option * opt;
   generic_option_gen_class option_gen;
   multiple_option_gen_class multi_gen;
+  multiple_option_new_list_node_gen_class multi_gen_node;
   string indent_str (indent, ' ');
 
   option_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
-  if (no_handle_error)
-    option_gen.set_gen_exit ("return");
-  else
-    option_gen.set_gen_exit ("exit");
-
   foropt 
     {
       if (opt->short_opt)
@@ -1202,6 +1400,20 @@ CmdlineParserCreator::generate_handle_option(ostream &stream,
 	      multi_gen.set_option_var_name (opt->var_arg);
 
 	      multi_gen.generate_multiple_option (stream, indent);
+
+              if (opt->type)
+                {
+                  ostringstream update_arg;
+
+                  do_update_arg (opt, update_arg, indent + 2, tmpvar);
+
+                  multi_gen_node.set_optarg(optarg);
+                  multi_gen_node.set_option_var_name (opt->var_arg);
+                  multi_gen_node.set_update_arg(update_arg.str());
+                  multi_gen_node.generate_multiple_option_new_list_node (stream, indent+2);
+                }
+              else
+                do_update_arg (opt, stream, indent + 2);
 	    }
 	  else
 	    {
@@ -1211,16 +1423,22 @@ CmdlineParserCreator::generate_handle_option(ostream &stream,
 	      option_gen.set_option_var_name (opt->var_arg);
 
 	      option_gen.generate_generic_option (stream, indent);
-	    }
-          generic_option_group_gen_class group_gen;
-          if (opt->group_value)
-            {
-              group_gen.set_group_var_name 
-                (canonize_name (opt->group_value));
-              group_gen.generate_generic_option_group (stream, indent);
+
+              // NOTE: I don't know why the following code was shared with the branch for
+              // multiple options because you can't specify multiple groupoption
+              generic_option_group_gen_class group_gen;
+              if (opt->group_value)
+                {
+                  group_gen.set_group_var_name 
+                    (canonize_name (opt->group_value));
+                  group_gen.generate_generic_option_group (stream, indent+2);
+                }
+
+              do_update_arg (opt, stream, indent + 2);
             }
 
-          do_update_arg (opt, stream, indent + 2);
+          if (opt->type != ARG_NO)
+            generateBreak(stream, indent + 2);
 
           stream << endl;
           stream << endl;
@@ -1350,13 +1568,49 @@ CmdlineParserCreator::generate_conf_parser(ostream &stream,
     return;
 
   stream << endl;
-  ConfigParserGenerator conf_parser_gen;
+  config_parser_source_gen_class conf_parser_gen;
   conf_parser_gen.set_parser_name (c_source_gen_class::parser_name);
-  conf_parser_gen.set_gen_exit (gen_exit);
+  conf_parser_gen.set_gen_exit (no_handle_error ? "return" : "exit");
   conf_parser_gen.set_package_var_name (c_source_gen_class::package_var_name);
+  conf_parser_gen.set_args_info (args_info_name);
+  conf_parser_gen.set_final_exit (c_source_gen_class::final_exit);
+
   conf_parser_gen.generate_config_parser_source (stream);
 }
 
+void
+CmdlineParserCreator::generate_free(ostream &stream,
+				    unsigned int indent)
+{
+  string indent_str (indent, ' ');
+  struct gengetopt_option * opt;
+
+  stream << endl;
+  stream << indent_str;
+
+  if (unamed_options || has_multiple_options_string ())
+    {
+      stream << "int i;";
+      stream << endl;
+      stream << indent_str;
+    }
+
+  foropt
+    {
+      free_option (opt, stream, indent);
+    }
+
+  if (unamed_options)
+    {
+      stream << endl;
+      stream << indent_str;
+
+      free_inputs_string_gen_class free_inputs_string;
+      free_inputs_string.set_structure (ARGS_STRUCT);
+      free_inputs_string.generate_free_inputs_string (stream, indent);
+    }
+}
+
 /*
   return a copy of the string passed after canonizing it (i.e. '-' and
   '.' are transformed in '_'.
@@ -1415,3 +1669,33 @@ canonize_name(const string &old)
   return canonized;
 }
 
+bool
+has_multiple_options_string()
+{
+  if (! has_multiple_options ())
+    return false;
+
+  struct gengetopt_option * opt;
+
+  foropt
+  {
+    if (opt->multiple && opt->type == ARG_STRING)
+      return true;
+  }
+
+  return false;
+}
+
+bool
+has_multiple_options()
+{
+  struct gengetopt_option * opt;
+
+  foropt
+  {
+    if (opt->multiple)
+      return true;
+  }
+
+  return false;
+}
