@@ -33,16 +33,19 @@ class CmdlineParserCreator : public header_gen_class, public c_source_gen_class
  protected:
   char *parser_function_name;
   char *filename;
+  char *args_info_name;
   char *header_filename;
   char *c_filename;
   string comment;
+  char *unamed_options;
 
-  bool unamed_options;
   bool long_help;
   bool no_handle_help;
   bool no_handle_version;
   bool no_handle_error;
   bool conf_parser;
+  bool gen_gengetopt_version;
+  bool gen_strdup;
 
   unsigned int tab_indentation ; /* tab indentation level */
 
@@ -65,7 +68,9 @@ class CmdlineParserCreator : public header_gen_class, public c_source_gen_class
     }
 
   void do_update_arg (struct gengetopt_option *opt, 
-                      ostream &stream, unsigned int indent);
+                      ostream &stream, unsigned int indent, const string argstr );
+  void free_option(struct gengetopt_option *opt, 
+                   ostream &stream, unsigned int indent);
 
   int generate_header_file();
   int generate_source();
@@ -82,7 +87,7 @@ class CmdlineParserCreator : public header_gen_class, public c_source_gen_class
   virtual void generate_clear_arg(ostream &stream, unsigned int indent);
   virtual void generate_given_init(ostream &stream, unsigned int indent);
   virtual void generate_struct_def(ostream &stream, unsigned int indent);
-  virtual void generate_multiple_fill_array(ostream &stream, unsigned int indent);
+
   virtual void generate_init_unamed(ostream &stream, unsigned int indent);
   virtual void generate_handle_help(ostream &stream, unsigned int indent);
   virtual void generate_handle_no_short_option(ostream &stream, 
@@ -99,19 +104,27 @@ class CmdlineParserCreator : public header_gen_class, public c_source_gen_class
   virtual void generate_strdup(ostream &stream, unsigned int indent);
   virtual void generate_handle_unamed(ostream &stream, unsigned int indent);
   virtual void generate_conf_parser(ostream &stream, unsigned int indent);
+  virtual void generate_reset_groups(ostream &stream, unsigned int indent);
 
-  void generateBreak(ostream &stream, unsigned int indent);
+  virtual void generate_free(ostream &stream, unsigned int indent);
+
+  void generateBreak(ostream &stream, unsigned int indent = 0);
 
  public:
 
-  CmdlineParserCreator (char *function_name, bool unamed_options,
+  CmdlineParserCreator (char *function_name, char *struct_name,
+                        char *unamed_options,
                         char *filename, char *header_ext, char *c_ext,
                         bool long_help, bool no_handle_help,
                         bool no_handle_version,
                         bool no_handle_error, bool conf_parser,
+                        bool gen_version,
                         const string &comment);
 
   int generate();
+
+  virtual void generate_list_def(ostream &stream, unsigned int indent);
+  virtual void generate_multiple_fill_array(ostream &stream, unsigned int indent);
 };
 
 #endif /* _GM_H */
