@@ -35,5 +35,10 @@ main (int argc, char **argv)
   for (i = 0; i < args_info.no_short_opt_given; ++i)
     printf ("passed no-short-opt: %s\n", args_info.no_short_opt_arg[i]);
 
+  printf ("noarg given %d times\n", args_info.noarg_given);
+  printf ("noarg_noshort given %d times\n", args_info.noarg_noshort_given);
+
+  test_multiple_cmd_parser_free (&args_info);
+
   return 0;
 }
